@@ -6,8 +6,8 @@ import numpy as np
 # local libraries
 from nion.typeshed import API_1_0 as API
 from nion.swift import Facade
+from nion.swift.model import Graphics
 from nion.swift.model import Symbolic
-from nion.utils import Binding
 
 from .DataCache import DataCache
 
@@ -15,6 +15,8 @@ _ = gettext.gettext
 
 
 class Map4DRGB:
+    attributes = {"connection_type": "map"}
+
     def __init__(self, computation, **kwargs):
         self.computation = computation
         self.__api = computation.api
@@ -181,8 +183,9 @@ class Map4DRGB:
     def execute(self, src, map_regions_r, map_regions_g, map_regions_b, gamma_r, gamma_g, gamma_b, enabled_r,
                 enabled_g, enabled_b):
         try:
-            data = self.computation.data_cache.get_cached_data(src)
-            rgb_data = np.zeros(src.xdata.data_shape[:2] + (3,), dtype=np.uint8)
+            src_data_item = src.data_item
+            data = self.computation.data_cache.get_cached_data(src_data_item)
+            rgb_data = np.zeros(src_data_item.xdata.data_shape[:2] + (3,), dtype=np.uint8)
             map_regions_rgb = (map_regions_b, map_regions_g, map_regions_r)
             channels_enabled = (enabled_b, enabled_g, enabled_r)
             gammas = (gamma_b, gamma_g, gamma_r)
@@ -190,9 +193,9 @@ class Map4DRGB:
                 if not channels_enabled[i]:
                     continue
                 map_regions = map_regions_rgb[i]
-                mask_data = np.zeros(src.xdata.data_shape[2:], dtype=np.bool)
+                mask_data = np.zeros(src_data_item.xdata.data_shape[2:], dtype=np.bool)
                 for region in map_regions:
-                    mask_data = np.logical_or(mask_data, region.get_mask(src.xdata.data_shape[2:]))
+                    mask_data = np.logical_or(mask_data, region.get_mask(src_data_item.xdata.data_shape[2:]))
                 if mask_data.any():
                     ind = np.arange(mask_data.size)[mask_data.ravel()]
                     new_data = np.sum(data[..., ind], axis=(-1))
@@ -205,10 +208,10 @@ class Map4DRGB:
                     rgb_data[..., i] = self.convert_to_8_bit(np.sum(data, axis=-1), gammas[i])
 
             self.__new_xdata = self.__api.create_data_and_metadata(rgb_data,
-                                                                   dimensional_calibrations=src.dimensional_calibrations[:2],
-                                                                   intensity_calibration=src.intensity_calibration)
-            metadata = src.metadata.copy()
-            metadata['nion.map_4d_rgb.parameters'] = {'src': src._data_item.write_to_dict(),
+                                                                   dimensional_calibrations=src_data_item.dimensional_calibrations[:2],
+                                                                   intensity_calibration=src_data_item.intensity_calibration)
+            metadata = src_data_item.metadata.copy()
+            metadata['nion.map_4d_rgb.parameters'] = {'src': src_data_item._data_item.write_to_dict(),
                                                       'map_regions_r': [region.write_to_dict() for region in map_regions_r],
                                                       'map_regions_g': [region.write_to_dict() for region in map_regions_g],
                                                       'map_regions_b': [region.write_to_dict() for region in map_regions_b],
@@ -239,35 +242,6 @@ class Map4DRGBMenuItem:
         self.__computation_data_items = dict()
         self.__tool_tip_boxes = list()
 
-        event_loop = self.__api.application._application.event_loop
-        def schedule_init():
-            if self.__api.application.document_controllers:
-                self.init()
-            else:
-                event_loop.call_later(0.5, schedule_init)
-        schedule_init()
-
-    def init(self):
-        document_model = self.__api.application._application.document_model
-        for computation in document_model.computations:
-            if computation.processing_id == 'nion.map_4d_rgb':
-                src = computation.get_input('src')
-                if hasattr(src, 'data_item') and src.data_item:
-                    src = src.data_item
-                if src:
-                    target = computation.get_output('target')
-                    if target is None:
-                        continue
-                    target_api = Facade.DataItem(target)
-                    pick_graphic = None
-                    for graphic in target_api.graphics:
-                        if graphic.label == 'Pick':
-                            pick_graphic = graphic
-                            break
-                    if pick_graphic is not None:
-                        display_item = self.__api.application._application.document_model.get_display_item_for_data_item(src)
-                        self.__connect_pick_graphic(pick_graphic, computation, display_item)
-
     def __display_item_changed(self, display_item):
         data_item = display_item.data_item if display_item else None
         if data_item:
@@ -295,34 +269,6 @@ class Map4DRGBMenuItem:
         #box = document_controller.show_tool_tip_box(text, timeout)
         self.__tool_tip_boxes.append(box)
 
-    def __connect_pick_graphic(self, pick_graphic, computation, display_item):
-        data_item = (display_item.data_items[0] if
-                     display_item and len(display_item.data_items) > 0 else None)
-        if data_item:
-            def update_collection_index_0(value):
-                collection_index = display_item.display_data_channel.collection_index
-                if value != collection_index[0]:
-                    display_item.display_data_channel.collection_index = (value, collection_index[1], 0)
-
-            def update_collection_index_1(value):
-                collection_index = display_item.display_data_channel.collection_index
-                if value != collection_index[1]:
-                    display_item.display_data_channel.collection_index = (collection_index[0], value, 0)
-
-            computation.pick_graphic_binding_0 = Binding.TuplePropertyBinding(pick_graphic._graphic, 'position', 0, converter=FloatTupleToIntTupleConverter(data_item.data.shape[0], 0))
-            computation.pick_graphic_binding_1 = Binding.TuplePropertyBinding(pick_graphic._graphic, 'position', 1, converter=FloatTupleToIntTupleConverter(data_item.data.shape[1], 1))
-            computation.pick_graphic_binding_0.target_setter = update_collection_index_0
-            computation.pick_graphic_binding_1.target_setter = update_collection_index_1
-            def collection_index_changed(key):
-                if key == 'collection_index':
-                    collection_index = display_item.display_data_channel.collection_index
-                    if int(pick_graphic.position[0]*data_item.data.shape[0]) != collection_index[0]:
-                        computation.pick_graphic_binding_0.update_source(collection_index)
-                    if int(pick_graphic.position[1]*data_item.data.shape[1]) != collection_index[1]:
-                        computation.pick_graphic_binding_1.update_source(collection_index)
-            computation.collection_index_changed_event_listener = display_item.display_data_channel.property_changed_event.listen(collection_index_changed)
-
-
     def menu_item_execute(self, window: API.DocumentWindow) -> None:
         document_controller = window._document_controller
         selected_display_item = document_controller.selected_display_item
@@ -335,42 +281,33 @@ class Map4DRGBMenuItem:
                 self.__show_tool_tips('wrong_shape')
                 return
             map_data_item = self.__api.library.create_data_item(title='Map 4D (RGB) of ' + data_item.title)
-            computation = self.__api.library.create_computation('nion.map_4d_rgb',
-                                                                inputs={'src': api_data_item,
-                                                                        'map_regions_r': [],
-                                                                        'map_regions_g': [],
-                                                                        'map_regions_b': [],
-                                                                        'gamma_r': 1.0,
-                                                                        'gamma_g': 1.0,
-                                                                        'gamma_b': 1.0,
-                                                                        'enabled_r': True,
-                                                                        'enabled_g': True,
-                                                                        'enabled_b': True},
-                                                                outputs={'target': map_data_item})
-            computation._computation.source = map_data_item._data_item
-
+            # the following uses internal API and should not be used as example code.
+            computation = document_controller.document_model.create_computation()
+            computation.create_input_item("src", Symbolic.make_item(selected_display_item.get_display_data_channel_for_data_item(data_item)))
+            computation.create_input_item("map_regions_r", Symbolic.make_item_list([]))
+            computation.create_input_item("map_regions_g", Symbolic.make_item_list([]))
+            computation.create_input_item("map_regions_b", Symbolic.make_item_list([]))
+            computation.create_variable("gamma_r", value_type="real", value=1.0)
+            computation.create_variable("gamma_g", value_type="real", value=1.0)
+            computation.create_variable("gamma_b", value_type="real", value=1.0)
+            computation.create_variable("enabled_r", value_type="boolean", value=True)
+            computation.create_variable("enabled_g", value_type="boolean", value=True)
+            computation.create_variable("enabled_b", value_type="boolean", value=True)
+            computation.processing_id = "nion.map_4d_rgb.2"
+            document_controller.document_model.set_data_item_computation(map_data_item._data_item, computation)
             map_display_item = document_controller.document_model.get_display_item_for_data_item(map_data_item._data_item)
             document_controller.show_display_item(map_display_item)
-            pick_graphic = map_data_item.add_point_region(0.5, 0.5)
-            pick_graphic.label = 'Pick'
-            self.__connect_pick_graphic(pick_graphic, computation, selected_display_item)
+            graphic = Graphics.PointGraphic()
+            graphic.label = "Pick"
+            graphic.role = "collection_index"
+            map_display_item.add_graphic(graphic)
+            # see note above.
             self.__computation_data_items.update({str(data_item.uuid): 'source',
                                                   str(map_data_item._data_item.uuid): 'map_4d'})
             self.__show_tool_tips()
             self.__display_item_changed_event_listener = (
                            document_controller.focused_display_item_changed_event.listen(self.__display_item_changed))
 
-class FloatTupleToIntTupleConverter:
-    def __init__(self, axis_size, axis_index):
-        self.axis_size = axis_size
-        self.axis_index = axis_index
-
-    def convert(self, value):
-        return int(value*self.axis_size)
-
-    def convert_back(self, value):
-        return (value[self.axis_index] + 0.5)/self.axis_size
-
 
 class Map4DExtension:
 
@@ -386,4 +323,4 @@ class Map4DExtension:
     def close(self):
         self.__menu_item_ref.close()
 
-Symbolic.register_computation_type('nion.map_4d_rgb', Map4DRGB)
+Symbolic.register_computation_type('nion.map_4d_rgb.2', Map4DRGB)
